From 7ba82ff640f7151d880a9d49666ea5c260a6d3b6 Mon Sep 17 00:00:00 2001
From: Ronan Dunklau <ronan.dunklau@aiven.io>
Date: Wed, 22 Dec 2021 12:07:57 +0100
Subject: [PATCH v1 2/2] Add malloc_stats extension to see the allocated memory
 at the end

---
 contrib/malloc_stats/Makefile       | 16 ++++++++++++
 contrib/malloc_stats/malloc_stats.c | 39 +++++++++++++++++++++++++++++
 2 files changed, 55 insertions(+)
 create mode 100644 contrib/malloc_stats/Makefile
 create mode 100644 contrib/malloc_stats/malloc_stats.c

diff --git a/contrib/malloc_stats/Makefile b/contrib/malloc_stats/Makefile
new file mode 100644
index 0000000000..b92063f5f5
--- /dev/null
+++ b/contrib/malloc_stats/Makefile
@@ -0,0 +1,16 @@
+# contrib/malloc_stats/Makefile
+
+MODULES = malloc_stats
+
+PGFILEDESC = "malloc_stats - register on_proc_exit to dump malloc_stats"
+
+ifdef USE_PGXS
+PG_CONFIG = pg_config
+PGXS := $(shell $(PG_CONFIG) --pgxs)
+include $(PGXS)
+else
+subdir = contrib/malloc_stats
+top_builddir = ../..
+include $(top_builddir)/src/Makefile.global
+include $(top_srcdir)/contrib/contrib-global.mk
+endif
diff --git a/contrib/malloc_stats/malloc_stats.c b/contrib/malloc_stats/malloc_stats.c
new file mode 100644
index 0000000000..802100b854
--- /dev/null
+++ b/contrib/malloc_stats/malloc_stats.c
@@ -0,0 +1,39 @@
+#include "postgres.h"
+#include "storage/ipc.h"
+#include "fmgr.h"
+#include "utils/builtins.h"
+
+#include "malloc.h"
+
+PG_MODULE_MAGIC;
+
+void _PG_init(void);
+void malloc_exit_callback(int, Datum);
+
+bool proc_exit_setup = false;
+
+void
+malloc_exit_callback(int code, Datum arg)
+{
+	struct mallinfo2 m;
+	m = mallinfo2();
+	elog(NOTICE, "On exit malloc stats: \n"
+		 		 "\tTotal memory: %ld\n"
+				 "\tMemory (without mmap): %ld\n"
+				 "\tMmaped memory: %ld\n"
+				 "\tIn-use: %ld\n"
+				 "\tFree blocks: %ld\n"
+				 "\tKeep cost: %ld",
+			 m.arena + m.hblkhd,
+			 m.arena,
+			 m.hblkhd,
+			 m.uordblks,
+			 m.fordblks,
+			 m.keepcost);
+}
+
+void
+_PG_init(void)
+{
+	on_proc_exit(malloc_exit_callback, 0);
+}
-- 
2.34.1

